#!/usr/bin/python
__author__ = 'Piotr Matuszczyk <piotr.matuszczyk@sap.com>'

import json, requests
from time import sleep

class Vrarestconnector:
  def __init__(self, username, password, vrahost, tenant, connection_timeout):
    self.username = username
    self.password = password
    self.vrahost = vrahost
    self.tenant = tenant
    self.connection_timeout = connection_timeout    
  token = None
  errorlist = [0]
  url = None
  payload = None
  response = None
  
  def checkResponseError(self, response):
    acceptedResponses = [200, 201]
    if not response.status_code in acceptedResponses:
      self.errorlist[0] = 1
      errtext = 'HTTP STATUS: '+str(response.status_code)+' for URL: '+response.url+'  '+response.text
      self.errorlist.append(errtext)
    else:
      self.errorlist[0] = 0
  
  def createToken(self):
    headers = {
    'Content-Type': 'application/json',
    'Accept': 'application/json'
    }
    payload = {"username": self.username, "password": self.password, "tenant": self.tenant}
    url = 'https://' + self.vrahost + '/identity/api/tokens'
    try:
      response = requests.post(url=url, data=json.dumps(payload), headers=headers, verify=False, timeout=self.connection_timeout)
    except requests.RequestException, e:
      self.errorlist[0] = 1
      self.errorlist.append(e)
    else:
      self.checkResponseError(response)

    if self.errorlist[0] == 0:
      self.token = 'Bearer ' + response.json()['id']
      
  def sendGETRequest(self, url):
    headers = {
    'Content-Type': 'application/json',
    'Accept': 'application/json',
    'Authorization': self.token
    }
    url = 'https://' + self.vrahost + url
    try:
      self.response = requests.get(url, headers=headers, verify=False, timeout=self.connection_timeout)
    except requests.RequestException, e:
      self.errorlist[0] = 1
      self.errorlist.append(e)
    else:
      self.checkResponseError(self.response)
      
  def sendPOSTRequest(self, o):
    headers = {
    'Content-Type': 'application/json',
    'Accept': 'application/json',
    'Authorization': self.token
    }
    url = 'https://' + self.vrahost + o.url
    try:
      self.response = requests.post(url, headers=headers, verify=False, data=json.dumps(o.payload), timeout=self.connection_timeout)
    except requests.RequestException, e:
      self.errorlist[0] = 1
      self.errorlist.append(e)
    else:
      self.checkResponseError(self.response)
      
  def findRequestId(self, o):
    o.requestid = self.response.headers['Location'].split('/')[-1]
    
  def checkRequestFinalStatus(self, o, wait_time):
    url = '/catalog-service/api/consumer/requests/'+o.requestid
    w = 0
    while w < wait_time:
      self.sendGETRequest(url)
      if self.errorlist[0] == 1:
        break
      w = w + 5
      if (self.response.json()['state'] == 'SUCCESSFUL' or self.response.json()['state'] == 'FAILED' or self.response.json()['state'] == 'PROVIDER_FAILED'):
        if (self.response.json()['state'] == 'FAILED' or self.response.json()['state'] == 'PROVIDER_FAILED'):
          o.errorlist[0] = 1
          o.errorlist.append(self.response.json()['requestCompletion']['completionDetails']+'   Request number: '+str(self.response.json()['requestNumber'])+' Request id: '+o.requestid)
          break
        break
      sleep(5)
    
    